@@ -64,8 +64,6 @@
 #include <syslog.h>
 #include <sys/time.h>
 #include <sys/stat.h>
-#include <sched.h>
-#include <limits.h>
 #include <ulogd/conffile.h>
 #include <ulogd/ulogd.h>
 #ifdef DEBUG
@@ -85,36 +83,13 @@ static char *ulogd_logfile = NULL;
 static const char *ulogd_configfile = ULOGD_CONFIGFILE;
 static const char *ulogd_pidfile = NULL;
 static int ulogd_pidfile_fd = -1;
-static FILE *syslog_dummy;
+static FILE syslog_dummy;
 
 static int info_mode = 0;
 
 static int verbose = 0;
 static int created_pidfile = 0;
 
-/*
- * Using self-pipe trick to handle signals safely.
- * (To avoid calling Async-Signal-Unsafe functions in signal handler.)
- *
- * cf. https://lwn.net/Articles/177897/
- *
- * Capacity of pipe(2) buffer is >= 4096.
- */
-static int signal_channel[2] = { -1, -1 };	/* initialize with invalid fd */
-#define	SIGNAL_RX_FD	(signal_channel[0])	/* main thread select() this fd */
-#define	SIGNAL_TX_FD	(signal_channel[1])	/* signal handler writes to this fd */
-
-static void signal_handler(int signal);
-static void sigterm_handler_task(int signal);
-static void signal_handler_task(int signal);
-
-static int  signal_channel_callback(int fd, unsigned int what, void *data);
-static struct ulogd_fd	signal_channel_ulogfd = {
-	.when = ULOGD_FD_READ,
-	.cb = &signal_channel_callback,
-	.data = &signal_channel_ulogfd,
-};
-
 /* linked list for all registered plugins */
 static LLIST_HEAD(ulogd_plugins);
 /* linked list for all plugins handle */
@@ -394,7 +369,6 @@ void ulogd_register_plugin(struct ulogd_plugin *me)
 	if (strcmp(me->version, VERSION)) { 
 		ulogd_log(ULOGD_NOTICE, 
 			  "plugin `%s' has incompatible version %s\n",
-			  me->name,
 			  me->version);
 		return;
 	}
@@ -405,87 +379,13 @@ void ulogd_register_plugin(struct ulogd_plugin *me)
 				  me->name);
 			exit(EXIT_FAILURE);
 		}
-		ulogd_log(ULOGD_NOTICE, "registering plugin `%s'\n", me->name);
+		ulogd_log(ULOGD_DEBUG, "registering plugin `%s'\n", me->name);
 		llist_add(&me->list, &ulogd_plugins);
 	} else {
 		get_plugin_infos(me);
 	}
 }
 
-/***********************************************************************
- * UTILITY FUNCTIONS FOR SIGNAL HANDLING
- ***********************************************************************/
-static void call_signal_handler_tasks(int sig)
-{
-	/* Deliver signals */
-	switch (sig) {
-	case SIGTERM:
-	case SIGINT:
-		sigterm_handler_task(sig);
-		break;
-	case SIGHUP:
-	case SIGALRM:
-	case SIGUSR1:
-	case SIGUSR2:
-		signal_handler_task(sig);
-		break;
-	default:
-		break;
-	}
-}
-
-static int signal_channel_callback(int fd, unsigned int what, void *data)
-{
-	int	ret;
-	unsigned char c;
-
-	while ((ret = read(fd, &c, 1)) > 0) {
-		call_signal_handler_tasks((int)c);
-	}
-	return 0;
-}
-
-static int create_signal_channel(void)
-{
-	int	ret;
-	long	flags;
-
-	if ((ret = pipe(signal_channel)) < 0)
-		return ret;
-
-	/* SIGNAL_RX_FD, channel[0] */
-	if ((flags = fcntl(SIGNAL_RX_FD, F_GETFL)) < 0)
-		goto err_exit;
-	flags |= O_NONBLOCK;
-	if ( fcntl(SIGNAL_RX_FD, F_SETFL, flags) < 0 )
-		goto err_exit;
-
-	/* SIGNAL_TX_FD, channel[1] */
-	if ((flags = fcntl(SIGNAL_TX_FD, F_GETFL)) < 0)
-		goto err_exit;
-	flags |= O_NONBLOCK;
-	if ( fcntl(SIGNAL_TX_FD, F_SETFL, flags) < 0 )
-		goto err_exit;
-
-	/* register SIGNAL_RX_FD to ulogd */
-	signal_channel_ulogfd.fd = SIGNAL_RX_FD;
-	ulogd_register_fd(&signal_channel_ulogfd);
-
-	return 0;
-
-err_exit:
-	if (SIGNAL_RX_FD >= 0) {
-		(void)close(SIGNAL_RX_FD);
-		SIGNAL_RX_FD = -1;
-	}
-	if (SIGNAL_TX_FD >= 0) {
-		(void)close(SIGNAL_TX_FD);
-		SIGNAL_TX_FD = -1;
-	}
-	return -1;
-}
-
-
 /***********************************************************************
  * MAIN PROGRAM
  ***********************************************************************/
@@ -527,7 +427,7 @@ void __ulogd_log(int level, char *file, int line, const char *format, ...)
 	if (level < loglevel_ce.u.value)
 		return;
 
-	if (logfile == syslog_dummy) {
+	if (logfile == &syslog_dummy) {
 		/* FIXME: this omits the 'file' string */
 		va_start(ap, format);
 		vsyslog(ulogd2syslog_level(level), format, ap);
@@ -542,7 +442,7 @@ void __ulogd_log(int level, char *file, int line, const char *format, ...)
 		timestr = ctime(&tm);
 		timestr[strlen(timestr)-1] = '\0';
 		fprintf(outfd, "%s <%1.1d> %s:%d ", timestr, level, file, line);
-		if (verbose && outfd != stderr)
+		if (verbose)
 			fprintf(stderr, "%s <%1.1d> %s:%d ", timestr, level, file, line);
 
 
@@ -552,7 +452,7 @@ void __ulogd_log(int level, char *file, int line, const char *format, ...)
 		/* flush glibc's buffer */
 		fflush(outfd);
 
-		if (verbose && outfd != stderr) {
+		if (verbose) {
 			va_start(ap, format);
 			vfprintf(stderr, format, ap);
 			va_end(ap);
@@ -661,16 +561,17 @@ pluginstance_alloc_init(struct ulogd_plugin *pl, char *pi_id,
 	}
 	size += pl->input.num_keys * sizeof(struct ulogd_key);
 	size += pl->output.num_keys * sizeof(struct ulogd_key);
-	pi = calloc(1, size);
+	pi = malloc(size);
 	if (!pi)
 		return NULL;
 
 	/* initialize */
+	memset(pi, 0, size);
 	INIT_LLIST_HEAD(&pi->list);
 	INIT_LLIST_HEAD(&pi->plist);
 	pi->plugin = pl;
 	pi->stack = stack;
-	strncpy(pi->id, pi_id, ULOGD_MAX_KEYLEN);
+	memcpy(pi->id, pi_id, sizeof(pi->id));
 
 	ptr = (void *)pi + sizeof(*pi);
 
@@ -728,41 +629,6 @@ static int load_plugin(const char *file)
 	return 0;
 }
 
-static int load_all_plugins(void)
-{
-	DIR *d;
-	struct dirent *dent;
-	char path[PATH_MAX];
-
-	d = opendir(ULOGD2_LIBDIR);
-	if (d == NULL) {
-		ulogd_log(ULOGD_ERROR, "load_all_plugins: opendir(%s): %s\n",
-			  ULOGD2_LIBDIR, strerror(errno));
-		return -1;
-	}
-
-	ulogd_log(ULOGD_NOTICE, "loading all plugins at %s\n", ULOGD2_LIBDIR);
-
-	while ((dent = readdir(d)) != NULL) {
-		if (strcmp(dent->d_name, ".") == 0 ||
-		    strcmp(dent->d_name, "..") == 0)
-			continue;
-
-		int len = strlen(dent->d_name);
-		if (len < 3)
-			continue;
-
-		if (strcmp(&dent->d_name[len - 3], ".so") != 0)
-			continue;
-
-		snprintf(path, sizeof(path), "%s/%s", ULOGD2_LIBDIR,
-			 dent->d_name);
-		if (load_plugin(path) != 0)
-			return -1;
-	}
-	return 0;
-}
-
 /* find an output key in a given stack, starting at 'start' */
 static struct ulogd_key *
 find_okey_in_stack(char *name,
@@ -960,10 +826,8 @@ static int create_stack(const char *option)
 	char *tok;
 	int ret;
 
-	if (llist_empty(&ulogd_plugins_handle))
-		load_all_plugins();
-
 	if (!buf) {
+		ulogd_log(ULOGD_ERROR, "");
 		ret = -ENOMEM;
 		goto out_buf;
 	}
@@ -1061,6 +925,7 @@ static void ulogd_main_loop(void)
 	struct timeval *next = NULL;
 
 	while (1) {
+		/* XXX: signal blocking? */
 		if (next != NULL && !timerisset(next))
 			next = ulogd_do_timer_run(&next_alarm);
 		else
@@ -1085,7 +950,7 @@ static int logfile_open(const char *name)
 		logfile = stdout;
 	} else if (!strcmp(name, "syslog")) {
 		openlog("ulogd", LOG_PID, LOG_DAEMON);
-		logfile = syslog_dummy = fopen("/dev/null", "w");
+		logfile = &syslog_dummy;
 	} else {
 		logfile = fopen(ulogd_logfile, "a");
 		if (!logfile) {
@@ -1360,19 +1225,7 @@ static void stop_stack()
 }
 
 
-/* This is a real SIGTERM, SIGINT, SIGHUP, SIGALRM, SIGUSR1, SIGUSR2 handler */
-static void signal_handler(int signal)
-{
-	unsigned char	c = (unsigned char)signal;
-
-	(void)write(SIGNAL_TX_FD, &c, 1);
-}
-
-/* This is NOT a real signal handler.
- * It is called in ulogd_main_loop() to avoid
- * calling Async-Signal-UnSafe functions in signal handler.
- */
-static void sigterm_handler_task(int signal)
+static void sigterm_handler(int signal)
 {
 
 	ulogd_log(ULOGD_NOTICE, "Terminal signal received, exiting\n");
@@ -1387,7 +1240,7 @@ static void sigterm_handler_task(int signal)
 	unload_plugins();
 #endif
 
-	if (logfile != NULL  && logfile != stdout) {
+	if (logfile != NULL  && logfile != stdout && logfile != &syslog_dummy) {
 		fclose(logfile);
 		logfile = NULL;
 	}
@@ -1402,26 +1255,21 @@ static void sigterm_handler_task(int signal)
 	exit(0);
 }
 
-
-/* This is NOT a real signal handler.
- * It is called in ulogd_main_loop() to avoid
- * calling Async-Signal-UnSafe functions in signal handler.
- */
-static void signal_handler_task(int signal)
+static void signal_handler(int signal)
 {
 	ulogd_log(ULOGD_NOTICE, "signal received, calling pluginstances\n");
 	
 	switch (signal) {
 	case SIGHUP:
 		/* reopen logfile */
-		if (logfile != stdout && logfile != syslog_dummy) {
+		if (logfile != stdout && logfile != &syslog_dummy) {
 			fclose(logfile);
 			logfile = fopen(ulogd_logfile, "a");
  			if (!logfile) {
 				fprintf(stderr, 
 					"ERROR: can't open logfile %s: %s\n", 
 					ulogd_logfile, strerror(errno));
-				sigterm_handler_task(signal);
+				sigterm_handler(signal);
 			}
 	
 		}
@@ -1433,19 +1281,6 @@ static void signal_handler_task(int signal)
 	deliver_signal_pluginstances(signal);
 }
 
-static void set_scheduler(void)
-{
-	struct sched_param schedparam;
-	int sched_type;
-
-	schedparam.sched_priority = sched_get_priority_max(SCHED_RR);
-	sched_type = SCHED_RR;
-
-	if (sched_setscheduler(0, sched_type, &schedparam) < 0)
-		fprintf(stderr, "WARNING: scheduler configuration failed:"
-			" %s\n", strerror(errno));
-}
-
 static void print_usage(void)
 {
 	printf("ulogd Version %s\n", VERSION);
@@ -1548,10 +1383,9 @@ int main(int argc, char* argv[])
 	}
 
 	/* command line has precedence on config file */
-	if (loglevel) {
+	if (loglevel)
 		loglevel_ce.u.value = loglevel;
 		loglevel_ce.flag |= CONFIG_FLAG_VAL_PROTECTED;
-	}
 
 	if (ulogd_pidfile) {
 		if (create_pidfile() < 0)
@@ -1567,8 +1401,8 @@ int main(int argc, char* argv[])
 
 	if (daemonize){
 		if (daemon(0, 0) < 0) {
-			ulogd_log(ULOGD_FATAL, "can't daemonize: %s (%d)\n",
-				  strerror(errno), errno);
+			ulogd_log(ULOGD_FATAL, "can't daemonize: %s (%d)",
+				  errno, strerror(errno));
 			warn_and_exit(daemonize);
 		}
 	}
@@ -1629,18 +1463,12 @@ int main(int argc, char* argv[])
 		}
 	}
 
-	if (create_signal_channel() < 0) {
-		ulogd_log(ULOGD_FATAL, "can't create signal channel\n");
-		warn_and_exit(daemonize);
-	}
-
-	signal(SIGTERM, &signal_handler);
-	signal(SIGINT, &signal_handler);
+	signal(SIGTERM, &sigterm_handler);
+	signal(SIGINT, &sigterm_handler);
 	signal(SIGHUP, &signal_handler);
 	signal(SIGALRM, &signal_handler);
 	signal(SIGUSR1, &signal_handler);
 	signal(SIGUSR2, &signal_handler);
-	set_scheduler();
 
 	ulogd_log(ULOGD_INFO, 
 		  "initialization finished, entering main loop\n");
@@ -1648,6 +1476,6 @@ int main(int argc, char* argv[])
 	ulogd_main_loop();
 
 	/* hackish, but result is the same */
-	sigterm_handler_task(SIGTERM);
+	sigterm_handler(SIGTERM);	
 	return(0);
 }
